package uk.ac.westminster.tic_tac_toe_self_learn_app

/* This represents a single square of the tic-tac-toe game */
class Cell {
    var entry: Char = ' '

    /* change the character in the square (cell)
       Returns true if success or false if changing the value failed
     */
    fun setValue(c: Char): Boolean {
        if (c == 'X' || c == 'O') {
            entry = c
            return true
        }
        else
            return false
    }
}


/* This represents the full grid of the game consisting of 3x3 cells */
class Board {

    // a 2-dimensional structure (list of lists of cells)
    var grid = listOf(listOf(Cell(), Cell(), Cell()),
                      listOf(Cell(), Cell(), Cell()),
                      listOf(Cell(), Cell(), Cell()),
    )

    /* This is a set of all squares which are currently empty in the grid
       The set contains elements of type (x, y). e.g. (2, 1) that means
       square in row 2 and col 1 is empty
     */
    var empty_cells = mutableSetOf<List<Int>>()

    init {
        for (x in 0..2)
            for (y in 0..2)
                empty_cells += listOf(x, y)
    }


    /* place char c at position (x,y) - player makes a move
       It returns true if the move succeeded or false otherwise */
    fun move(c: Char, x: Int, y: Int): Boolean {
        if (x in 0..2 && y in 0..2) {
            // update both structures to indicate that position (x, y) is taken
            empty_cells -= listOf(x, y)
            return grid[x][y].setValue(c)
        }
        else  // invalid move
            return false
    }

}